@@ -1,8 +1,10 @@
+import contextlib
 import functools
 
 import onnx.helper
 import torch
 import torch.nn as nn
+import torch.onnx
 import torch.onnx.symbolic_registry as sym_reg
 
 
@@ -47,7 +49,7 @@ class _AnnotationInit(object):
     def _edit_attr(self, node, found_idx):
         start_idx = found_idx + self.len_wrap_func_name
         next_ub_idx = node.doc_string[start_idx:].find('_')
-        tracked_id = node.doc_string[start_idx:start_idx+next_ub_idx]
+        tracked_id = node.doc_string[start_idx:start_idx + next_ub_idx]
         assert tracked_id.isdigit()
 
         attrs = [onnx.helper.make_attribute(k, v) for k, v in
@@ -102,7 +104,7 @@ class _AnnotationInit(object):
 
         start_idx = found_idx + self.len_anchor_func_name
         end_idx = node.doc_string[start_idx:].find('__')
-        track_info = node.doc_string[start_idx:start_idx+end_idx].split('_')
+        track_info = node.doc_string[start_idx:start_idx + end_idx].split('_')
         assert len(track_info) == 3
         start_end, tracked_id, node_num = track_info
 
@@ -163,6 +165,12 @@ class _Annotation(object):
             child_module.forward = self.original_forwards[name]
 
 
+@contextlib.contextmanager
+def _nullcontext():
+    # contextlib.nullcontext equivalent, needed for Python 3.6 support.
+    yield
+
+
 def annotate(**attrs):
     """Annotation parameters to the target function.
 
@@ -191,7 +199,9 @@ def annotate(**attrs):
     Args:
         attrs (dict): annotation parameters
     """
-    return _Annotation(**attrs)
+    if torch.onnx.is_in_onnx_export():
+        return _Annotation(**attrs)
+    return _nullcontext()
 
 
 def apply_annotation(fn, *args, **attrs):
@@ -253,7 +263,7 @@ def apply_annotation(fn, *args, **attrs):
 
 class _Anchor(_Annotation):
 
-    def __init__(self,  **attrs):
+    def __init__(self, **attrs):
         super(_Anchor, self).__init__(**attrs)
         self.called_count = -1
         self.started = False
@@ -364,4 +374,6 @@ def scoped_anchor(**attrs):
     Args:
         attrs (dict): annotation parameters
     """
-    return _Anchor(**attrs)
+    if torch.onnx.is_in_onnx_export():
+        return _Anchor(**attrs)
+    return _nullcontext()
