@@ -1,45 +1,43 @@
 import collections
 import contextlib
 import copy
-import os
 import time
+from typing import Any, Callable, Dict, Generator, List, Optional, Union
+from typing import TYPE_CHECKING
 import warnings
 
 import torch
 
 from pytorch_pfn_extras import writing
-from pytorch_pfn_extras.reporting import Reporter
+from pytorch_pfn_extras import reporting
 from pytorch_pfn_extras.training import extension as extension_module
 from pytorch_pfn_extras.training import trigger as trigger_module
-from pytorch_pfn_extras.training import util as util_module
+from pytorch_pfn_extras.training import _util as util_module
 
-# Select the best-resolution timer function
-try:
-    _get_time = time.perf_counter
-except AttributeError:
-    if os.name == 'nt':
-        _get_time = time.clock
-    else:
-        _get_time = time.time
+_get_time = time.perf_counter
 
 
 class _ExtensionEntry:
 
-    def __init__(self, extension, priority, trigger, call_before_training):
+    def __init__(
+            self,
+            extension: extension_module.Extension,
+            priority: int,
+            trigger: trigger_module.Trigger,
+            call_before_training: bool
+    ) -> None:
         self.extension = extension
         self.trigger = trigger
         self.priority = priority
         self.call_before_training = call_before_training
 
-    def state_dict(self):
+    def state_dict(self) -> Dict[str, Any]:
         state = {}
-        if hasattr(self.extension, 'state_dict'):
-            state['extension'] = self.extension.state_dict()
-        if hasattr(self.trigger, 'state_dict'):
-            state['trigger'] = self.trigger.state_dict()
+        state['extension'] = self.extension.state_dict()
+        state['trigger'] = self.trigger.state_dict()
         return state
 
-    def load_state_dict(self, to_load):
+    def load_state_dict(self, to_load: Dict[str, Any]) -> None:
         if 'extension' in to_load:
             self.extension.load_state_dict(to_load['extension'])
         if 'trigger' in to_load:
@@ -53,13 +51,14 @@ class _BaseExtensionsManager:
 
     def __init__(
             self,
-            models,
-            optimizers,
-            max_epochs,
-            extensions,
-            out_dir,
-            writer,
-            stop_trigger=None):
+            models: Union[torch.nn.Module, Dict[str, torch.nn.Module]],
+            optimizers: Union[torch.optim.Optimizer, Dict[str, torch.optim.Optimizer]],
+            max_epochs: int,
+            extensions: Optional[List['extension_module.ExtensionLike']],
+            out_dir: str,
+            writer: Optional[writing.Writer],
+            stop_trigger: 'trigger_module.TriggerLike' = None
+    ) -> None:
         if extensions is None:
             extensions = []
         if stop_trigger is None:
@@ -72,10 +71,10 @@ class _BaseExtensionsManager:
             writer = writing.SimpleWriter(out_dir=out_dir)
         # triggers are stateful, so we need to make a copy for internal use
         self._internal_stop_trigger = copy.deepcopy(self._stop_trigger)
-        self.observation = {}
+        self.observation: Dict[str, reporting.ReportValue] = {}
         self._out = out_dir
         self.writer = writer
-        self.reporter = Reporter()
+        self.reporter = reporting.Reporter()
         self._start_extensions_called = False
 
         if not isinstance(models, dict):
@@ -99,8 +98,9 @@ class _BaseExtensionsManager:
         self.max_epochs = max_epochs
         self._start_iteration = 0
         # Defer!
-        self._start_time = None
-        self._extensions = collections.OrderedDict()
+        self._start_time: Optional[float] = None
+        self._iters_per_epoch: Optional[int] = None
+        self._extensions: Dict[str, _ExtensionEntry] = collections.OrderedDict()
         for ext in extensions:
             self.extend(ext)
 
@@ -111,60 +111,64 @@ class _BaseExtensionsManager:
     # the snapshot extension must run first to restore the training state.
 
     @property
-    def iteration(self):
+    def iteration(self) -> int:
         self.start_extensions()
         return self._iteration
 
     @iteration.setter
-    def iteration(self, value):
+    def iteration(self, value: int) -> None:
         self._iteration = value
 
     @property
-    def models(self):
+    def models(self) -> Dict[str, torch.nn.Module]:
         self.start_extensions()
         return self._models
 
     @property
-    def optimizers(self):
+    def optimizers(self) -> Dict[str, torch.optim.Optimizer]:
         self.start_extensions()
         return self._optimizers
 
     @property
-    def elapsed_time(self):
-        # Unavailable until the initial run_iteration call
-        return _get_time()-self._start_time
+    def elapsed_time(self) -> float:
+        if self._start_time is None:
+            raise RuntimeError(
+                'Unavailable until the initial run_iteration call.')
+        return _get_time() - self._start_time
 
     @property
-    def is_before_training(self):
+    def is_before_training(self) -> bool:
         # Extensions will start via self.iteration
         return self.iteration == 0
 
     @property
-    def epoch(self):
+    def epoch(self) -> int:
         # Extensions will start via self.iteration
+        assert self._iters_per_epoch is not None
         return self.iteration // self._iters_per_epoch
 
     @property
-    def epoch_detail(self):
+    def epoch_detail(self) -> float:
         # Extensions will start via self.iteration
+        assert self._iters_per_epoch is not None
         return self.iteration / self._iters_per_epoch
 
     @property
-    def stop_trigger(self):
+    def stop_trigger(self) -> bool:
         self.start_extensions()
         # Trigger is stateful, we close the extensions the first time
         # it evaluates to True, as it won't do it again
         return self._stop_trigger(self)
 
     @property
-    def out(self):
+    def out(self) -> str:
         if self.writer.out_dir is not None:
             return self.writer.out_dir
         else:
             return self._out
 
     @property
-    def updater(self):
+    def updater(self) -> '_BaseExtensionsManager':
         warnings.warn(
             'The `updater` attribute has been deprecated in v0.3.0.'
             ' Use `iteration`, `epoch`, and `epoch_detail` attributes in'
@@ -175,13 +179,17 @@ class _BaseExtensionsManager:
             ' `snapshot_iter_{.iteration}`).', DeprecationWarning)
         return self
 
-    def _prepare_for_training(self, start_iteration, iters_per_epoch):
+    def _prepare_for_training(
+            self,
+            start_iteration: int,
+            iters_per_epoch: int
+    ) -> None:
         self.iteration = start_iteration
         self._iters_per_epoch = iters_per_epoch
 
-    def start_extensions(self):
+    def start_extensions(self) -> None:
         if self._start_extensions_called:
-            # This method must not be called twice or more.
+            # Extensions are already started or during the initialization.
             return
         else:
             self._start_extensions_called = True
@@ -195,20 +203,28 @@ class _BaseExtensionsManager:
 
         # invoke initializer of each extension
         for _, entry in self.extensions:
-            initializer = getattr(entry.extension, 'initialize', None)
+            initializer = entry.extension.initialize
             finished = getattr(entry.trigger, 'finished', False)
-            if initializer and not finished:
+            if not finished:
                 initializer(self)
 
         # call extensions before training loop
         self.observation = {}
         with self.reporter.scope(self.observation):
-            for name, entry in self.extensions:
+            for _, entry in self.extensions:
                 if entry.call_before_training:
                     entry.extension(self)
 
-    def extend(self, extension, name=None, trigger=None, priority=None,
-               *, call_before_training=False, **kwargs):
+    def extend(
+            self,
+            extension: 'extension_module.ExtensionLike',
+            name: Optional[str] = None,
+            trigger: 'trigger_module.TriggerLike' = None,
+            priority: Optional[int] = None,
+            *,
+            call_before_training: bool = False,
+            **kwargs: Dict[str, Any],
+    ) -> None:
         """Registers an extension to the manager.
 
         :class:`Extension` is a callable object which is called after each
@@ -247,25 +263,19 @@ class _BaseExtensionsManager:
                 instead.
 
         """
+        ext = extension_module._as_extension(extension)
         if name is None:
-            name = getattr(extension, 'name', None)
-            if name is None:
-                name = getattr(extension, 'default_name', None)
-                if name is None:
-                    name = getattr(extension, '__name__', None)
-                    if name is None:
-                        raise TypeError('name is not given for the extension')
+            name = ext.name or ext.default_name
         if name == 'training':
             raise ValueError(
                 'the name "training" is prohibited as an extension name')
 
         if trigger is None:
-            trigger = getattr(extension, 'trigger', (1, 'iteration'))
+            trigger = ext.trigger
         trigger = trigger_module.get_trigger(trigger)
 
         if priority is None:
-            priority = getattr(
-                extension, 'priority', extension_module.PRIORITY_READER)
+            priority = ext.priority
 
         modified_name = name
         ordinal = 0
@@ -273,11 +283,11 @@ class _BaseExtensionsManager:
             ordinal += 1
             modified_name = '%s_%d' % (name, ordinal)
 
-        extension.name = modified_name
+        ext.name = modified_name
         self._extensions[modified_name] = _ExtensionEntry(
-            extension, priority, trigger, call_before_training)
+            ext, priority, trigger, call_before_training)
 
-    def get_extension(self, name):
+    def get_extension(self, name: str) -> extension_module.Extension:
         """Returns the extension of a given name.
 
         Args:
@@ -293,34 +303,57 @@ class _BaseExtensionsManager:
         else:
             raise ValueError('extension %s not found' % name)
 
-    def run_extensions(self):
-        for name, entry in self.extensions:
+    def run_extensions(self) -> None:
+        to_run = []
+        for _, entry in self.extensions:
             if entry.trigger(self):
-                entry.extension(self)
+                # Execution of snapshot extensions are deferred until all the
+                # triggers are evaluated.
+                # If we don't do this, when two (or more) snapshot extensions
+                # are registered and triggers for them are stateful, the first
+                # snapshot extension will save the state of the second trigger
+                # before invoking it although it will be executed later in this
+                # iteration, making them to fire again just after resuming from
+                # the snaphsot saved by the first snapshot extension.
+                # Non-snapshot extensions are executed right away (note that
+                # the order is already sorted by the priority) as they will
+                # report values that might be needed by other triggers, i.e.,
+                # trigger based on evaluator reported value.
+                if entry.priority == extension_module.PRIORITY_SNAPSHOT:
+                    to_run.append(entry.extension)
+                else:
+                    entry.extension(self)
+        for extension in to_run:
+            extension(self)
 
-    def _finalize_extensions(self):
+    def _finalize_extensions(self) -> None:
         for _, entry in self.extensions:
             # Some mock objects for tests give errors
             # if we use `getattr`
             try:
-                if getattr(entry.extension, 'finalize'):
+                if entry.extension.finalize:
                     entry.extension.finalize()
             except AttributeError:
                 pass
 
-    def state_dict(self, *, transform_models=lambda n, x: x):
+    def state_dict(
+            self,
+            *,
+            transform_models: Callable[
+                [str, torch.nn.Module], torch.nn.Module] = lambda n, x: x
+    ) -> Dict[str, Any]:
         """
         transform_models is a function that apply a transformation
         to a model.
 
         When using a `torch.nn.DataParallel` model, if we want
-        to save only yhe `.module` object, state_dict can be
-        called as follows
-        state_dict(transform_models=lambda n, x: x.module)
+        to save only the `.module` object, state_dict can be
+        called as follows:
+
+        >>> manager.state_dict(transform_models=lambda n, x: x.module)
         """
-        to_save = {}
+        to_save: Dict[str, Any] = {}
         to_save['_start_iteration'] = self.iteration
-        # Save manager status ?
         to_save['models'] = {
             name: transform_models(name, self._models[name]).state_dict()
             for name in self._models}
@@ -330,17 +363,25 @@ class _BaseExtensionsManager:
                                  for name in self._extensions}
         return to_save
 
-    def load_state_dict(self, to_load, *, transform_models=lambda n, x: x):
+    def load_state_dict(
+            self,
+            to_load: Dict[str, Any],
+            *,
+            transform_models: Callable[
+                [str, torch.nn.Module], torch.nn.Module] = lambda n, x: x
+    ) -> None:
         """
         transform_models is a function that apply a transformation
-        to a model before loading its state
+        to a model before loading its state.
 
         When using a `torch.nn.DataParallel` model, if we want
         to load the original state in a model with the
-        `torch.nn.DataParallel` applied
-        load_state_dict(
-            state, transform_models=(lambda n, x:
-                x.module if isinstance(x, torch.nn.DataParallel) else  x))
+        `torch.nn.DataParallel` applied:
+
+        >>> manager.load_state_dict(
+                state, transform_models=(
+                    lambda n, x: x.module
+                    if isinstance(x, torch.nn.DataParallel) else x))
         """
         self._start_iteration = to_load['_start_iteration']
         self.iteration = self._start_iteration
@@ -379,15 +420,16 @@ class ExtensionsManager(_BaseExtensionsManager):
 
     def __init__(
             self,
-            models,
-            optimizers,
-            max_epochs,
+            models: Union[torch.nn.Module, Dict[str, torch.nn.Module]],
+            optimizers: Union[torch.optim.Optimizer, Dict[str, torch.optim.Optimizer]],
+            max_epochs: int,
             *,
-            iters_per_epoch,
-            extensions=None,
-            out_dir='result',
-            stop_trigger=None,
-            writer=None):
+            iters_per_epoch: Optional[int],
+            extensions: Optional[List['extension_module.ExtensionLike']] = None,
+            out_dir: str = 'result',
+            stop_trigger: 'trigger_module.TriggerLike' = None,
+            writer: Optional[writing.Writer] = None
+    ) -> None:
         super().__init__(
             models, optimizers, max_epochs, extensions,
             out_dir, writer, stop_trigger)
@@ -398,8 +440,12 @@ class ExtensionsManager(_BaseExtensionsManager):
         self._prepare_for_training(0, iters_per_epoch)
 
     @contextlib.contextmanager
-    def run_iteration(self, *, step_optimizers=None):
-        """ Context manager to run an iteration.
+    def run_iteration(
+            self,
+            *,
+            step_optimizers: Optional[List[str]] = None
+    ) -> Generator[None, None, None]:
+        """Context manager to run an iteration.
 
         This manager can additionally run a step in the
         specified optimizers names.
@@ -423,9 +469,8 @@ class ExtensionsManager(_BaseExtensionsManager):
                 for name in step_optimizers_names:
                     self._optimizers[name].step()
             finally:
-                # In chainer, the iteration count was increased
-                # just before calling the extensions, we need
-                # to keep the semantics
+                # The iteration count is increased just before calling the
+                # extensions.
                 self.iteration += 1
                 self.run_extensions()
 
@@ -433,11 +478,15 @@ class ExtensionsManager(_BaseExtensionsManager):
             self._finalize_extensions()
 
 
+if TYPE_CHECKING:
+    import ignite
+
+
 class IgniteExtensionsManager(_BaseExtensionsManager):
     """Manages extensions and the current status in Ignite training loop.
 
     Args:
-        engine (ignite.Engine): Ignite trainer engine
+        engine (ignite.engine.Engine): Ignite trainer engine
         models (dict or torch.nn.Module): Map of string to Module
             or an actual Module
         optimizers (dict or torch.Optimizer): Map of string to Optimizer
@@ -450,15 +499,18 @@ class IgniteExtensionsManager(_BaseExtensionsManager):
     """
     def __init__(
             self,
-            engine,
-            models,
-            optimizers,
-            max_epochs,
+            engine: 'ignite.engine.Engine',
+            models: Union[torch.nn.Module, Dict[str, torch.nn.Module]],
+            optimizers: Union[torch.optim.Optimizer, Dict[str, torch.optim.Optimizer]],
+            max_epochs: int,
             *,
-            extensions=None,
-            out_dir='result',
-            writer=None):
+            extensions: Optional[List['extension_module.ExtensionLike']] = None,
+            out_dir: str = 'result',
+            writer: Optional[writing.Writer] = None
+    ) -> None:
         import ignite
+        if not isinstance(engine, ignite.engine.Engine):
+            raise TypeError("Argument 'engine' must be of ignite.Engine type.")
         if (util_module._get_ignite_version(ignite.__version__)
                 < util_module._get_ignite_version('0.3.0')):
             raise ImportError('Ignite version found {}. '
@@ -469,18 +521,19 @@ class IgniteExtensionsManager(_BaseExtensionsManager):
         self._start_epoch = 0  # Used to correctly restore snapshots
         self.set_ignite_handlers()
 
-    def set_ignite_handlers(self):
+    def set_ignite_handlers(self) -> None:
+        from ignite.engine import Engine
         from ignite.engine import Events
 
         # Set a handler that sets the reporter scope on every iteration
         @self.engine.on(Events.ITERATION_STARTED)
-        def set_reporter_on_iter(engine):
+        def set_reporter_on_iter(engine: Engine) -> None:
             self.observation = {}
             self.cm = self.reporter.scope(self.observation)
             self.cm.__enter__()
 
         @self.engine.on(Events.STARTED)
-        def set_training_started(engine):
+        def set_training_started(engine: Engine) -> None:
             iters_per_epoch = len(engine.state.dataloader)
             # Initialize manager once before extensions' `initialize` call
             self._prepare_for_training(0, iters_per_epoch)
@@ -495,25 +548,36 @@ class IgniteExtensionsManager(_BaseExtensionsManager):
             # Make all the next
             # handlers to be executed after user defined ones
             @self.engine.on(Events.ITERATION_COMPLETED)
-            def run_extensions_on_iter(engine):
+            def run_extensions_on_iter(engine: Engine) -> None:
                 self.iteration = engine.state.iteration
                 self.run_extensions()
 
             # This should be the last extension to be run
             @self.engine.on(Events.ITERATION_COMPLETED)
-            def close_reporter_on_iter(engine):
+            def close_reporter_on_iter(engine: Engine) -> None:
                 self.cm.__exit__(None, None, None)
 
         @self.engine.on(Events.COMPLETED)
-        def set_extensions_cleanup(engine):
+        def set_extensions_cleanup(engine: Engine) -> None:
             self._finalize_extensions()
 
-    def state_dict(self):
-        to_save = super().state_dict()
+    def state_dict(
+            self,
+            *,
+            transform_models: Callable[
+                [str, torch.nn.Module], torch.nn.Module] = lambda n, x: x
+    ) -> Dict[str, Any]:
+        to_save = super().state_dict(transform_models=transform_models)
         to_save['_epoch_length'] = self.engine.state.epoch_length
         to_save['_start_iteration'] = self.engine.state.iteration
         return to_save
 
-    def load_state_dict(self, to_load):
-        super().load_state_dict(to_load)
+    def load_state_dict(
+            self,
+            to_load: Dict[str, Any],
+            *,
+            transform_models: Callable[
+                [str, torch.nn.Module], torch.nn.Module] = lambda n, x: x
+    ) -> None:
+        super().load_state_dict(to_load, transform_models=transform_models)
         self._start_epoch = self._start_iteration // to_load['_epoch_length']
