@@ -1,4 +1,7 @@
-class IntervalTrigger:
+from pytorch_pfn_extras.training import trigger
+
+
+class IntervalTrigger(trigger.Trigger):
 
     """Trigger based on a fixed interval.
 
