@@ -1,4 +1,7 @@
-class ManualScheduleTrigger:
+from pytorch_pfn_extras.training import trigger
+
+
+class ManualScheduleTrigger(trigger.Trigger):
 
     """Trigger invoked at specified point(s) of iterations or epochs.
 
